# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import backups as schema
from tempest.lib.api_schema.response.volume.v3_64 import backups as schemav364
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.volume import base_client


class BackupsClient(base_client.BaseClient):
    """Volume V3 Backups client"""

    schema_versions_info = [
        {'min': None, 'max': '3.63', 'schema': schema},
        {'min': '3.64', 'max': None, 'schema': schemav364}
        ]

    def create_backup(self, **kwargs):
        """Creates a backup of volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#create-a-backup
        """
        post_body = json.dumps({'backup': kwargs})
        resp, body = self.post('backups', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_backup(self, backup_id, **kwargs):
        """Updates the specified volume backup.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#update-a-backup
        """
        put_body = json.dumps({'backup': kwargs})
        resp, body = self.put('backups/%s' % backup_id, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def restore_backup(self, backup_id, **kwargs):
        """Restore volume from backup.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#restore-a-backup
        """
        post_body = json.dumps({'restore': kwargs})
        resp, body = self.post('backups/%s/restore' % (backup_id), post_body)
        body = json.loads(body)
        self.validate_response(schema.restore_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_backup(self, backup_id):
        """Delete a backup of volume."""
        resp, body = self.delete('backups/%s' % backup_id)
        self.validate_response(schema.delete_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_backup(self, backup_id):
        """Returns the details of a single backup."""
        url = "backups/%s" % backup_id
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_backups(self, detail=False, **params):
        """List all the tenant's backups.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-backups-for-project
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-backups-with-detail
        """
        url = "backups"
        schema = self.get_schema(self.schema_versions_info)
        list_backups_schema = schema.list_backups_no_detail
        if detail:
            url += "/detail"
            list_backups_schema = schema.list_backups_with_detail
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(list_backups_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def export_backup(self, backup_id):
        """Export backup metadata record."""
        url = "backups/%s/export_record" % backup_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.export_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def import_backup(self, **kwargs):
        """Import backup metadata record.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#import-a-backup
        """
        post_body = json.dumps({'backup-record': kwargs})
        resp, body = self.post("backups/import_record", post_body)
        body = json.loads(body)
        self.validate_response(schema.import_backup, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reset_backup_status(self, backup_id, status):
        """Reset the specified backup's status."""
        post_body = json.dumps({'os-reset_status': {"status": status}})
        resp, body = self.post('backups/%s/action' % backup_id, post_body)
        self.validate_response(schema.reset_backup_status, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_backup(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'backup'



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from oslo_log import log as logging
from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


CONF = config.CONF
LOG = logging.getLogger(__name__)


class ImageCachingTest(base.BaseV2ImageTest):
    """Here we test the caching operations for image"""
    credentials = ['primary', 'admin']

    def setUp(self):
        super(ImageCachingTest, self).setUp()
        # NOTE(abhishekk): As caching is enabled instance boot or volume
        # boot or image download can also cache image, so we are going to
        # maintain our caching information to avoid disturbing other tests
        self.cached_info = []
        self.cached_info_remote = []

    def tearDown(self):
        # Delete all from cache/queue if we exit abruptly
        for image_id in self.cached_info:
            self.os_admin.image_cache_client.cache_delete(image_id)

        for image_id in self.cached_info_remote:
            self.os_admin.image_cache_client.cache_delete(image_id)

        super(ImageCachingTest, self).tearDown()

    @classmethod
    def skip_checks(cls):
        super(ImageCachingTest, cls).skip_checks()
        # Check to see if we should even be running these tests.
        if not CONF.image.image_caching_enabled:
            raise cls.skipException('Target system is not configured with '
                                    'glance caching')

    def image_create_and_upload(self, upload=True, **kwargs):
        """Wrapper that returns a test image."""
        if 'name' not in kwargs:
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__name__ + "-image")
            kwargs['name'] = name

        params = dict(kwargs)
        image = self.create_image(**params)
        self.assertEqual('queued', image['status'])
        if not upload:
            return image

        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.store_image_file(image['id'], image_file)

        image = self.client.show_image(image['id'])
        return image

    def _assertCheckCache(self, cached_images, cached):
        cached_list = []
        for image in cached_images:
            cached_list.append(image['image_id'])

        for image in cached:
            self.assertIn(image, cached_list)

    @decorators.idempotent_id('4bf6adba-2f9f-47e9-a6d5-37f21ad4387c')
    def test_image_caching_cycle(self):
        """Test image cache APIs"""
        # Ensure that non-admin user is not allowed to perform caching
        # operations
        self.assertRaises(lib_exc.Forbidden,
                          self.os_primary.image_cache_client.list_cache)

        # Check there is nothing cached by us
        output = self.os_admin.image_cache_client.list_cache()
        self._assertCheckCache(output['cached_images'], self.cached_info)

        # Non-existing image should raise NotFound exception
        self.assertRaises(lib_exc.NotFound,
                          self.os_admin.image_cache_client.cache_queue,
                          'non-existing-image-id')

        # Verify that we can not use queued image for queueing
        image = self.image_create_and_upload(name='queued', upload=False)
        self.assertRaises(lib_exc.BadRequest,
                          self.os_admin.image_cache_client.cache_queue,
                          image['id'])

        # Create one image
        image = self.image_create_and_upload(name='first',
                                             container_format='bare',
                                             disk_format='raw',
                                             visibility='private')
        self.assertEqual('active', image['status'])

        # Queue image for caching
        self.os_admin.image_cache_client.cache_queue(image['id'])
        # Wait for image caching
        LOG.info("Waiting for image %s to get cached", image['id'])
        caching = waiters.wait_for_caching(
            self.client,
            self.os_admin.image_cache_client,
            image['id'])

        self.cached_info.append(image['id'])
        # verify that we have image cached
        self._assertCheckCache(caching['cached_images'], self.cached_info)

        # Verify that we can delete images from caching and queueing with
        # api call.
        self.os_admin.image_cache_client.cache_clear()
        output = self.os_admin.image_cache_client.list_cache()
        self.assertEqual(0, len(output['queued_images']))
        self.assertEqual(0, len(output['cached_images']))

        # Verify that invalid header value for target returns 400 response
        self.assertRaises(lib_exc.BadRequest,
                          self.os_admin.image_cache_client.cache_clear,
                          target="invalid")
        # Remove all data from local information
        self.cached_info = []

    @decorators.idempotent_id('0a6b7e10-bc30-4a41-91ff-69fb4f5e65f2')
    def test_remote_and_self_cache(self):
        """Test image cache works with self and remote glance service"""
        if not CONF.image.alternate_image_endpoint:
            raise self.skipException('No image_remote service to test '
                                     'against')

        # Check there is nothing is cached by us on current and
        # remote node
        output = self.os_admin.image_cache_client.list_cache()
        self._assertCheckCache(output['cached_images'], self.cached_info)

        output = self.os_admin.cache_client_remote.list_cache()
        self._assertCheckCache(output['cached_images'],
                               self.cached_info_remote)

        # Create one image
        image = self.image_create_and_upload(name='first',
                                             container_format='bare',
                                             disk_format='raw',
                                             visibility='private')
        self.assertEqual('active', image['status'])

        # Queue image for caching on local node
        self.os_admin.image_cache_client.cache_queue(image['id'])
        # Wait for image caching
        LOG.info("Waiting for image %s to get cached", image['id'])
        caching = waiters.wait_for_caching(
            self.client,
            self.os_admin.image_cache_client,
            image['id'])
        self.cached_info.append(image['id'])
        # verify that we have image in cache on local node
        self._assertCheckCache(caching['cached_images'], self.cached_info)
        # verify that we don't have anything cached on remote node
        output = self.os_admin.cache_client_remote.list_cache()
        self._assertCheckCache(output['cached_images'],
                               self.cached_info_remote)

        # cache same image on remote node
        self.os_admin.cache_client_remote.cache_queue(image['id'])
        # Wait for image caching
        LOG.info("Waiting for image %s to get cached", image['id'])
        caching = waiters.wait_for_caching(
            self.client,
            self.os_admin.cache_client_remote,
            image['id'])
        self.cached_info_remote.append(image['id'])

        # verify that we have image cached on remote node
        output = self.os_admin.cache_client_remote.list_cache()
        self._assertCheckCache(output['cached_images'],
                               self.cached_info_remote)

        # Verify that we can delete image from remote cache and it
        # still present in local cache
        self.os_admin.cache_client_remote.cache_clear()
        output = self.os_admin.cache_client_remote.list_cache()
        self.assertEqual(0, len(output['queued_images']))
        self.assertEqual(0, len(output['cached_images']))

        output = self.os_admin.image_cache_client.list_cache()
        self._assertCheckCache(output['cached_images'], self.cached_info)

        # Delete image from local cache as well
        self.os_admin.image_cache_client.cache_clear()
        output = self.os_admin.image_cache_client.list_cache()
        self.assertEqual(0, len(output['queued_images']))
        self.assertEqual(0, len(output['cached_images']))

        # Remove all data from local and remote information
        self.cached_info = []
        self.cached_info_remote = []



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_71 import servers as servers271


###########################################################################
#
# 2.73:
#
# The locked_reason parameter is now returned in the response body of the
# following calls:
#
# - POST /servers/{server_id}/action (where the action is rebuild)
# - PUT /servers/{server_id} (update)
# - GET /servers/{server_id} (show)
# - GET /servers/detail (list)
#
###########################################################################

# The "locked_reason" parameter will either be a string or None.
locked_reason = {'type': ['string', 'null']}

rebuild_server = copy.deepcopy(servers271.rebuild_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'locked_reason': locked_reason})
rebuild_server['response_body']['properties']['server'][
    'required'].append('locked_reason')

rebuild_server_with_admin_pass = copy.deepcopy(
    servers271.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'locked_reason': locked_reason})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('locked_reason')

update_server = copy.deepcopy(servers271.update_server)
update_server['response_body']['properties']['server'][
    'properties'].update({'locked_reason': locked_reason})
update_server['response_body']['properties']['server'][
    'required'].append('locked_reason')

get_server = copy.deepcopy(servers271.get_server)
get_server['response_body']['properties']['server'][
    'properties'].update({'locked_reason': locked_reason})
get_server['response_body']['properties']['server'][
    'required'].append('locked_reason')

list_servers_detail = copy.deepcopy(servers271.list_servers_detail)
list_servers_detail['response_body']['properties']['servers']['items'][
    'properties'].update({'locked_reason': locked_reason})
list_servers_detail['response_body']['properties']['servers']['items'][
    'required'].append('locked_reason')

# NOTE(lajoskatona): Below are the unchanged schema in this microversion. We
# need to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.71 ***
list_servers = copy.deepcopy(servers271.list_servers)
show_server_diagnostics = copy.deepcopy(servers271.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers271.get_remote_consoles)
list_tags = copy.deepcopy(servers271.list_tags)
update_all_tags = copy.deepcopy(servers271.update_all_tags)
delete_all_tags = copy.deepcopy(servers271.delete_all_tags)
check_tag_existence = copy.deepcopy(servers271.check_tag_existence)
update_tag = copy.deepcopy(servers271.update_tag)
delete_tag = copy.deepcopy(servers271.delete_tag)
attach_volume = copy.deepcopy(servers271.attach_volume)
show_volume_attachment = copy.deepcopy(servers271.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers271.list_volume_attachments)
show_instance_action = copy.deepcopy(servers271.show_instance_action)
create_backup = copy.deepcopy(servers271.create_backup)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumesNegativeTest(base.BaseV2ComputeTest):
    """Negative tests of volumes with compute microversion less than 2.36"""

    # These tests will fail with a 404 starting from microversion 2.36. For
    # more information, see:
    # https://docs.openstack.org/api-ref/compute/#volume-extension-os-volumes-os-snapshots-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(VolumesNegativeTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(VolumesNegativeTest, cls).setup_clients()
        cls.client = cls.volumes_extensions_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c03ea686-905b-41a2-8748-9635154b7c57')
    def test_volume_get_nonexistent_volume_id(self):
        """Test getting details of a non existent volume should fail"""
        # Creating a nonexistent volume id
        # Trying to GET a non existent volume
        self.assertRaises(lib_exc.NotFound, self.client.show_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('54a34226-d910-4b00-9ef8-8683e6c55846')
    def test_volume_delete_nonexistent_volume_id(self):
        """Test deleting a nonexistent volume should fail"""
        # Creating nonexistent volume id
        # Trying to DELETE a non existent volume
        self.assertRaises(lib_exc.NotFound, self.client.delete_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5125ae14-152b-40a7-b3c5-eae15e9022ef')
    def test_create_volume_with_invalid_size(self):
        """Test creating volume with invalid size should fail"""
        v_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Volume')
        metadata = {'Type': 'work'}
        self.assertRaises(lib_exc.BadRequest, self.client.create_volume,
                          size='#$%', display_name=v_name, metadata=metadata)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('131cb3a1-75cc-4d40-b4c3-1317f64719b0')
    def test_create_volume_without_passing_size(self):
        """Test creating volume without specifying size should fail"""
        v_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Volume')
        metadata = {'Type': 'work'}
        self.assertRaises(lib_exc.BadRequest, self.client.create_volume,
                          size='', display_name=v_name, metadata=metadata)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8cce995e-0a83-479a-b94d-e1e40b8a09d1')
    def test_create_volume_with_size_zero(self):
        """Test creating volume with size=0 should fail"""
        v_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Volume')
        metadata = {'Type': 'work'}
        self.assertRaises(lib_exc.BadRequest, self.client.create_volume,
                          size='0', display_name=v_name, metadata=metadata)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('62bab09a-4c03-4617-8cca-8572bc94af9b')
    def test_get_volume_without_passing_volume_id(self):
        """Test getting volume details without volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.show_volume, '')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('62972737-124b-4513-b6cf-2f019f178494')
    def test_delete_invalid_volume_id(self):
        """Test deleting volume with an invalid volume id should fail"""
        self.assertRaises(
            lib_exc.NotFound,
            self.client.delete_volume,
            data_utils.rand_name(prefix=CONF.resource_name_prefix,
                                 name='invalid'))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0d1417c5-4ae8-4c2c-adc5-5f0b864253e5')
    def test_delete_volume_without_passing_volume_id(self):
        """Test deleting volume without volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_volume, '')



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib import decorators


class VolumeHostsAdminTestsJSON(base.BaseVolumeAdminTest):
    """Test fetching volume hosts info by admin users"""

    @decorators.idempotent_id('d5f3efa2-6684-4190-9ced-1c2f526352ad')
    def test_list_hosts(self):
        """Test listing volume hosts"""
        hosts = self.admin_hosts_client.list_hosts()['hosts']
        self.assertGreaterEqual(len(hosts), 2,
                                "The count of volume hosts is < 2, "
                                "response of list hosts is: %s" % hosts)

    @decorators.idempotent_id('21168d57-b373-4b71-a3ac-f2c88f0c5d31')
    def test_show_host(self):
        """Test getting volume host details"""
        hosts = self.admin_hosts_client.list_hosts()['hosts']
        self.assertGreaterEqual(len(hosts), 2,
                                "The count of volume hosts is < 2, "
                                "response of list hosts is: %s" % hosts)

        # Note(jeremyZ): The show host API is to show volume usage info on the
        # specified cinder-volume host. If the host does not run cinder-volume
        # service, or the cinder-volume service is disabled on the host, the
        # show host API should fail (return code: 404). The cinder-volume host
        # is presented in format: <host-name>@driver-name.
        c_vol_hosts = [host['host_name'] for host in hosts
                       if (host['service'] == 'cinder-volume' and
                           host['service-state'] == 'enabled')]
        self.assertNotEmpty(c_vol_hosts,
                            "No available cinder-volume host is found, "
                            "all hosts that found are: %s" % hosts)

        # Check each cinder-volume host.
        for host in c_vol_hosts:
            host_details = self.admin_hosts_client.show_host(host)['host']
            self.assertNotEmpty(host_details)



# Copyright 2013 IBM Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class HypervisorAdminTestBase(base.BaseV2ComputeAdminTest):
    """Tests Hypervisors API that require admin privileges"""

    @classmethod
    def setup_clients(cls):
        super(HypervisorAdminTestBase, cls).setup_clients()
        cls.client = cls.os_admin.hypervisor_client

    def _list_hypervisors(self):
        # List of hypervisors
        hypers = self.client.list_hypervisors()['hypervisors']
        return hypers


class HypervisorAdminTestJSON(HypervisorAdminTestBase):
    """Tests Hypervisors API that require admin privileges"""

    @decorators.idempotent_id('7f0ceacd-c64d-4e96-b8ee-d02943142cc5')
    def test_get_hypervisor_list(self):
        """List of hypervisor and available hypervisors hostname"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers, "No hypervisors found.")

    @decorators.idempotent_id('1e7fdac2-b672-4ad1-97a4-bad0e3030118')
    def test_get_hypervisor_list_details(self):
        """Display the details of the all hypervisor"""
        hypers = self.client.list_hypervisors(detail=True)['hypervisors']
        self.assertNotEmpty(hypers, "No hypervisors found.")

    @decorators.idempotent_id('94ff9eae-a183-428e-9cdb-79fde71211cc')
    def test_get_hypervisor_show_details(self):
        """Display the details of the specified hypervisor"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers, "No hypervisors found.")

        details = self.client.show_hypervisor(hypers[0]['id'])['hypervisor']
        self.assertNotEmpty(details)
        self.assertEqual(details['hypervisor_hostname'],
                         hypers[0]['hypervisor_hostname'])

    @decorators.idempotent_id('797e4f28-b6e0-454d-a548-80cc77c00816')
    def test_get_hypervisor_stats(self):
        """Verify the stats of the all hypervisor"""
        stats = (self.client.show_hypervisor_statistics()
                 ['hypervisor_statistics'])
        self.assertNotEmpty(stats)

    @decorators.idempotent_id('91a50d7d-1c2b-4f24-b55a-a1fe20efca70')
    def test_get_hypervisor_uptime(self):
        """Verify that GET shows the specified hypervisor uptime"""
        hypers = self._list_hypervisors()

        # Ironic will register each baremetal node as a 'hypervisor',
        # so the hypervisor list can contain many hypervisors of type
        # 'ironic'. If they are ALL ironic, skip this test since ironic
        # doesn't support hypervisor uptime. Otherwise, remove them
        # from the list of hypervisors to test.
        ironic_only = True
        hypers_without_ironic = []
        for hyper in hypers:
            details = (self.client.show_hypervisor(hyper['id'])
                       ['hypervisor'])
            if (details['hypervisor_type'] != 'ironic' and
                    details['state'] == 'up'):
                hypers_without_ironic.append(hyper)
                ironic_only = False

        if ironic_only:
            raise self.skipException(
                "Ironic does not support hypervisor uptime")

        has_valid_uptime = False
        for hyper in hypers_without_ironic:
            # because hypervisors might be disabled, this loops looking
            # for any good hit.
            try:
                uptime = (self.client.show_hypervisor_uptime(hyper['id'])
                          ['hypervisor'])
                if uptime:
                    has_valid_uptime = True
                    break
            except Exception:
                pass
        self.assertTrue(
            has_valid_uptime,
            "None of the hypervisors had a valid uptime: %s" % hypers)


class HypervisorAdminV228Test(HypervisorAdminTestBase):
    """Tests Hypervisors API higher than 2.27 that require admin privileges"""

    min_microversion = '2.28'

    @decorators.idempotent_id('d46bab64-0fbe-4eb8-9133-e6ee56188cc5')
    def test_get_list_hypervisor_details(self):
        """Test listing and showing hypervisor details"""
        # NOTE(zhufl): This test tests the hypervisor APIs response schema
        # for 2.28 microversion. No specific assert or behaviour verification
        # is needed.
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers, "No hypervisors found.")
        self.client.show_hypervisor(hypers[0]['id'])


class HypervisorAdminUnderV252Test(HypervisorAdminTestBase):
    """Tests Hypervisors API below 2.53 that require admin privileges"""

    max_microversion = '2.52'

    @decorators.idempotent_id('e81bba3f-6215-4e39-a286-d52d2f906862')
    def test_get_hypervisor_show_servers(self):
        """Test showing instances about the specific hypervisors"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers, "No hypervisors found.")

        hostname = hypers[0]['hypervisor_hostname']
        hypervisors = (self.client.list_servers_on_hypervisor(hostname)
                       ['hypervisors'])
        self.assertNotEmpty(hypervisors)

    @decorators.idempotent_id('d7e1805b-3b14-4a3b-b6fd-50ec6d9f361f')
    def test_search_hypervisor(self):
        """Test searching for hypervisors by its name"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers, "No hypervisors found.")
        hypers = self.client.search_hypervisor(
            hypers[0]['hypervisor_hostname'])['hypervisors']
        self.assertNotEmpty(hypers, "No hypervisors found.")


class HypervisorAdminV253TestBase(base.BaseV2ComputeAdminTest):
    """Tests Hypervisors API above 2.53 that require admin privileges"""

    min_microversion = '2.53'

    @classmethod
    def setup_clients(cls):
        super(HypervisorAdminV253TestBase, cls).setup_clients()
        cls.client = cls.os_admin.hypervisor_client

    @decorators.idempotent_id('4ab54a14-77a2-4e39-b9d2-1306d157c705')
    def test_list_show_detail_hypervisors(self):
        """Verify the list, list details, and show hypevisors

        This verify the Hypervisor API response schema with v2.53 microversion
        """
        self.client.list_hypervisors(
            detail=True, with_servers=True)['hypervisors']
        hypers = self.client.list_hypervisors(with_servers=True)['hypervisors']
        self.client.show_hypervisor(
            hypers[0]['id'], with_servers=True)['hypervisor']



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.object_storage import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ContainerTest(base.BaseObjectTest):
    """Test containers"""

    @classmethod
    def resource_cleanup(cls):
        cls.delete_containers()
        super(ContainerTest, cls).resource_cleanup()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('92139d73-7819-4db1-85f8-3f2f22a8d91f')
    def test_create_container(self):
        """Test creating container"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        resp, _ = self.container_client.update_container(container_name)
        self.containers.append(container_name)
        self.assertHeaders(resp, 'Container', 'PUT')

    @decorators.idempotent_id('49f866ed-d6af-4395-93e7-4187eb56d322')
    def test_create_container_overwrite(self):
        """Test overwriting container with the same name"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        self.container_client.update_container(container_name)
        self.containers.append(container_name)

        resp, _ = self.container_client.update_container(container_name)
        self.assertHeaders(resp, 'Container', 'PUT')

    @decorators.idempotent_id('c2ac4d59-d0f5-40d5-ba19-0635056d48cd')
    def test_create_container_with_metadata_key(self):
        """Test creating container with the blank value of metadata"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        headers = {'X-Container-Meta-test-container-meta': ''}
        resp, _ = self.container_client.update_container(
            container_name,
            **headers)
        self.containers.append(container_name)
        self.assertHeaders(resp, 'Container', 'PUT')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        # if the value of metadata is blank, metadata is not registered
        # in the server
        self.assertNotIn('x-container-meta-test-container-meta', resp)

    @decorators.idempotent_id('e1e8df32-7b22-44e1-aa08-ccfd8d446b58')
    def test_create_container_with_metadata_value(self):
        """Test creating container with metadata value"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')

        # metadata name using underscores should be converted to hyphens
        headers = {'X-Container-Meta-test_container_meta': 'Meta1'}
        resp, _ = self.container_client.update_container(
            container_name,
            **headers)
        self.containers.append(container_name)
        self.assertHeaders(resp, 'Container', 'PUT')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertIn('x-container-meta-test-container-meta', resp)
        self.assertEqual(resp['x-container-meta-test-container-meta'],
                         headers['X-Container-Meta-test_container_meta'])

    @decorators.idempotent_id('24d16451-1c0c-4e4f-b59c-9840a3aba40e')
    def test_create_container_with_remove_metadata_key(self):
        """Test creating container with the blank value of remove metadata"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        headers = {'X-Container-Meta-test-container-meta': 'Meta1'}
        self.container_client.update_container(container_name, **headers)
        self.containers.append(container_name)

        headers = {'X-Remove-Container-Meta-test-container-meta': ''}
        resp, _ = self.container_client.update_container(
            container_name,
            **headers)
        self.assertHeaders(resp, 'Container', 'PUT')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertNotIn('x-container-meta-test-container-meta', resp)

    @decorators.idempotent_id('8a21ebad-a5c7-4e29-b428-384edc8cd156')
    def test_create_container_with_remove_metadata_value(self):
        """Test creating container with remove metadata"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        headers = {'X-Container-Meta-test-container-meta': 'Meta1'}
        self.container_client.update_container(container_name, **headers)
        self.containers.append(container_name)
        headers = {'X-Remove-Container-Meta-test-container-meta': 'Meta1'}
        resp, _ = self.container_client.update_container(
            container_name,
            **headers)
        self.assertHeaders(resp, 'Container', 'PUT')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertNotIn('x-container-meta-test-container-meta', resp)

    @decorators.idempotent_id('95d3a249-b702-4082-a2c4-14bb860cf06a')
    def test_delete_container(self):
        """Test deleting container"""
        # create a container
        container_name = self.create_container()
        # delete container, success asserted within
        resp, _ = self.container_client.delete_container(container_name)
        self.assertHeaders(resp, 'Container', 'DELETE')

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('312ff6bd-5290-497f-bda1-7c5fec6697ab')
    def test_list_container_contents(self):
        """Test getting container contents list"""
        container_name = self.create_container()
        object_name, _ = self.create_object(container_name)

        resp, object_list = self.container_client.list_container_objects(
            container_name)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.idempotent_id('4646ac2d-9bfb-4c7d-a3c5-0f527402b3df')
    def test_list_container_contents_with_no_object(self):
        """Test getting empty container contents list"""
        container_name = self.create_container()

        resp, object_list = self.container_client.list_container_objects(
            container_name)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEmpty(object_list)

    @decorators.idempotent_id('fe323a32-57b9-4704-a996-2e68f83b09bc')
    def test_list_container_contents_with_delimiter(self):
        """Test getting container contents list using delimiter param"""
        container_name = self.create_container()
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject/')
        self.create_object(container_name, object_name)

        params = {'delimiter': '/'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name.split('/')[0] + '/'], object_list)

    @decorators.idempotent_id('55b4fa5c-e12e-4ca9-8fcf-a79afe118522')
    def test_list_container_contents_with_end_marker(self):
        """Test getting container contents list using end_marker param"""
        container_name = self.create_container()
        object_name, _ = self.create_object(container_name)

        params = {'end_marker': object_name + 'zzzz'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.idempotent_id('196f5034-6ab0-4032-9da9-a937bbb9fba9')
    def test_list_container_contents_with_format_json(self):
        """Test getting container contents list using format_json param"""
        container_name = self.create_container()
        self.create_object(container_name)

        params = {'format': 'json'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')

        self.assertIsNotNone(object_list)
        self.assertTrue([c['name'] for c in object_list])
        self.assertTrue([c['hash'] for c in object_list])
        self.assertTrue([c['bytes'] for c in object_list])
        self.assertTrue([c['content_type'] for c in object_list])
        self.assertTrue([c['last_modified'] for c in object_list])

    @decorators.idempotent_id('655a53ca-4d15-408c-a377-f4c6dbd0a1fa')
    def test_list_container_contents_with_format_xml(self):
        """Test getting container contents list using format_xml param"""
        container_name = self.create_container()
        self.create_object(container_name)

        params = {'format': 'xml'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')

        self.assertIsNotNone(object_list)
        self.assertEqual(object_list.tag, 'container')
        self.assertIn('name', object_list.keys())
        self.assertEqual(object_list.find(".//object").tag, 'object')
        self.assertEqual(object_list.find(".//name").tag, 'name')
        self.assertEqual(object_list.find(".//hash").tag, 'hash')
        self.assertEqual(object_list.find(".//bytes").tag, 'bytes')
        self.assertEqual(object_list.find(".//content_type").tag,
                         'content_type')
        self.assertEqual(object_list.find(".//last_modified").tag,
                         'last_modified')

    @decorators.idempotent_id('297ec38b-2b61-4ff4-bcd1-7fa055e97b61')
    def test_list_container_contents_with_limit(self):
        """Test getting container contents list using limit param"""
        container_name = self.create_container()
        object_name, _ = self.create_object(container_name)

        params = {'limit': data_utils.rand_int_id(1, 10000)}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.idempotent_id('c31ddc63-2a58-4f6b-b25c-94d2937e6867')
    def test_list_container_contents_with_marker(self):
        """Test getting container contents list using marker param"""
        container_name = self.create_container()
        object_name, _ = self.create_object(container_name)

        params = {'marker': 'AaaaObject1234567890'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.idempotent_id('58ca6cc9-6af0-408d-aaec-2a6a7b2f0df9')
    def test_list_container_contents_with_path(self):
        """Test getting container contents list using path param"""
        container_name = self.create_container()
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        object_name = 'Swift/' + object_name
        self.create_object(container_name, object_name)

        params = {'path': 'Swift'}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.idempotent_id('77e742c7-caf2-4ec9-8aa4-f7d509a3344c')
    def test_list_container_contents_with_prefix(self):
        """Test getting container contents list using prefix param"""
        container_name = self.create_container()
        object_name, _ = self.create_object(container_name)

        prefix_key = object_name[0:8]
        params = {'prefix': prefix_key}
        resp, object_list = self.container_client.list_container_objects(
            container_name,
            params=params)
        self.assertHeaders(resp, 'Container', 'GET')
        self.assertEqual([object_name], object_list)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('96e68f0e-19ec-4aa2-86f3-adc6a45e14dd')
    def test_list_container_metadata(self):
        """Test listing container metadata"""
        container_name = self.create_container()

        metadata = {'name': 'Pictures'}
        self.container_client.create_update_or_delete_container_metadata(
            container_name,
            create_update_metadata=metadata)

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertHeaders(resp, 'Container', 'HEAD')
        self.assertIn('x-container-meta-name', resp)
        self.assertEqual(resp['x-container-meta-name'], metadata['name'])

    @decorators.idempotent_id('a2faf936-6b13-4f8d-92a2-c2278355821e')
    def test_list_no_container_metadata(self):
        """Test listing container without metadata"""
        container_name = self.create_container()

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertHeaders(resp, 'Container', 'HEAD')
        self.assertNotIn('x-container-meta-', str(resp))

    @decorators.idempotent_id('cf19bc0b-7e16-4a5a-aaed-cb0c2fe8deef')
    def test_update_container_metadata_with_create_and_delete_metadata(self):
        """Test updating container with adding and deleting metadata

        Send one request of adding and deleting metadata.
        """
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        metadata_1 = {'X-Container-Meta-test-container-meta1': 'Meta1'}
        self.container_client.update_container(container_name, **metadata_1)
        self.containers.append(container_name)

        metadata_2 = {'test-container-meta2': 'Meta2'}
        resp, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                container_name,
                create_update_metadata=metadata_2,
                delete_metadata={'test-container-meta1': 'Meta1'}))
        self.assertHeaders(resp, 'Container', 'POST')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertNotIn('x-container-meta-test-container-meta1', resp)
        self.assertIn('x-container-meta-test-container-meta2', resp)
        self.assertEqual(resp['x-container-meta-test-container-meta2'],
                         metadata_2['test-container-meta2'])

    @decorators.idempotent_id('2ae5f295-4bf1-4e04-bfad-21e54b62cec5')
    def test_update_container_metadata_with_create_metadata(self):
        """Test updating container metadata using add metadata"""
        container_name = self.create_container()

        metadata = {'test-container-meta1': 'Meta1'}
        resp, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                container_name,
                create_update_metadata=metadata))
        self.assertHeaders(resp, 'Container', 'POST')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertIn('x-container-meta-test-container-meta1', resp)
        self.assertEqual(resp['x-container-meta-test-container-meta1'],
                         metadata['test-container-meta1'])

    @decorators.idempotent_id('3a5ce7d4-6e4b-47d0-9d87-7cd42c325094')
    def test_update_container_metadata_with_delete_metadata(self):
        """Test updating container metadata using delete metadata"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        metadata = {'X-Container-Meta-test-container-meta1': 'Meta1'}
        self.container_client.update_container(container_name, **metadata)
        self.containers.append(container_name)

        resp, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                container_name,
                delete_metadata={'test-container-meta1': 'Meta1'}))
        self.assertHeaders(resp, 'Container', 'POST')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertNotIn('x-container-meta-test-container-meta1', resp)

    @decorators.idempotent_id('31f40a5f-6a52-4314-8794-cd89baed3040')
    def test_update_container_metadata_with_create_metadata_key(self):
        """Test updating container metadata with a blank value of metadata"""
        container_name = self.create_container()

        metadata = {'test-container-meta1': ''}
        resp, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                container_name,
                create_update_metadata=metadata))
        self.assertHeaders(resp, 'Container', 'POST')

        resp, _ = self.container_client.list_container_metadata(
            container_name)
        self.assertNotIn('x-container-meta-test-container-meta1', resp)

    @decorators.idempotent_id('a2e36378-6f1f-43f4-840a-ffd9cfd61914')
    def test_update_container_metadata_with_delete_metadata_key(self):
        """Test updating container metadata with a blank value of metadata"""
        container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        headers = {'X-Container-Meta-test-container-meta1': 'Meta1'}
        self.container_client.update_container(container_name, **headers)
        self.containers.append(container_name)

        metadata = {'test-container-meta1': ''}
        resp, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                container_name,
                delete_metadata=metadata))
        self.assertHeaders(resp, 'Container', 'POST')

        resp, _ = self.container_client.list_container_metadata(container_name)
        self.assertNotIn('x-container-meta-test-container-meta1', resp)



# Copyright 2023 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ImageTaskCreate(base.BaseV2ImageAdminTest):
    """Test image task operations"""

    @classmethod
    def skip_checks(cls):
        # TODO(msava): Add additional skipcheck with task conversion_format and
        # glance ceph backend then will be available
        # in tempest image service config options.
        super(ImageTaskCreate, cls).skip_checks()
        if not CONF.image.http_image:
            skip_msg = ("%s skipped as http_image is not available " %
                        cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ImageTaskCreate, cls).resource_setup()

    @staticmethod
    def _prepare_image_tasks_param(type="import",
                                   disk_format=['qcow2'],
                                   image_from_format=['qcow2'],
                                   image_location=CONF.image.http_image):
        # TODO(msava): Need to add additional disk formats then
        # task conversion_format and glance Ceph backend will be
        # available in image service options
        """Prepare image task params.
        By default, will create task type 'import'

        The same index is used for both params and creates a task
        :param type Type of the task.
        :param disk_format: Each format in the list is a different task.
        :param image_from_format: Each format in the list is a different task.
        :param image_location Location to import image from.
        :return: A list with all task.
        """
        i = 0
        tasks = list()
        while i < len(disk_format):
            image_name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix, name="task_image")
            image_property = {"container_format": "bare",
                              "disk_format": disk_format[0],
                              "visibility": "public",
                              "name": image_name
                              }
            task = {
                "type": type,
                "input": {
                    "image_properties": image_property,
                    "import_from_format": image_from_format[0],
                    "import_from": image_location
                }
            }
            tasks.append(task)
            i += 1
        return tasks

    def _verify_disk_format(self, task_body):
        expected_disk_format = \
            task_body['input']['image_properties']['disk_format']
        image_id = task_body['result']['image_id']
        observed_disk_format = self.admin_client.show_image(
            image_id)['disk_format']
        # If glance backend storage is Ceph glance will convert
        # image to raw format.
        # TODO(msava): Need to change next lines once task conversion_format
        # and glance ceph backend will be available in image service options
        if observed_disk_format == 'raw':
            return
        self.assertEqual(observed_disk_format, expected_disk_format,
                         message="Expected disk format not match ")

    @decorators.skip_because(bug='2030527')
    @decorators.idempotent_id('669d5387-0340-4abf-b62d-7cc89f539c8c')
    def test_image_tasks_create(self):
        """Test task type 'import' image """

        # Prepare params for task type 'import'
        tasks = self._prepare_image_tasks_param()

        # Create task type 'import'
        body = self.os_admin.tasks_client.create_task(**tasks[0])
        task_id = body['id']
        task_body = waiters.wait_for_tasks_status(self.os_admin.tasks_client,
                                                  task_id, 'success')
        self.addCleanup(self.admin_client.delete_image,
                        task_body['result']['image_id'])
        task_image_id = task_body['result']['image_id']
        waiters.wait_for_image_status(self.client, task_image_id, 'active')
        self._verify_disk_format(task_body)

        # Verify disk format
        image_body = self.client.show_image(task_image_id)
        task_disk_format = \
            task_body['input']['image_properties']['disk_format']
        image_disk_format = image_body['disk_format']
        self.assertEqual(
            image_disk_format, task_disk_format,
            message="Image Disc format %s not match to expected %s"
                    % (image_disk_format, task_disk_format))

    @decorators.idempotent_id("ad6450c6-7060-4ee7-a2d1-41c2604b446c")
    @decorators.attr(type=['negative'])
    def test_task_create_fake_image_location(self):
        kwargs = {
            'prefix': CONF.resource_name_prefix,
            'name': 'dummy-img-file'
        }
        http_fake_url = ''.join(
            ["http://", data_utils.rand_name(**kwargs), ".qcow2"])
        task = self._prepare_image_tasks_param(
            image_from_format=['qcow2'],
            disk_format=['qcow2'],
            image_location=http_fake_url)
        body = self.os_admin.tasks_client.create_task(**task[0])
        task_observed = \
            waiters.wait_for_tasks_status(self.os_admin.tasks_client,
                                          body['id'], 'failure')
        task_observed = task_observed['status']
        self.assertEqual(task_observed, 'failure')



# Copyright 2013 NEC Corporation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class ServicesAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Nova Services API.

    List and Enable/Disable require admin privileges.
    """

    @classmethod
    def setup_clients(cls):
        super(ServicesAdminTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.services_client

    @decorators.idempotent_id('5be41ef4-53d1-41cc-8839-5c2a48a1b283')
    def test_list_services(self):
        """Listing nova services"""
        services = self.client.list_services()['services']
        self.assertNotEmpty(services)

    @decorators.idempotent_id('f345b1ec-bc6e-4c38-a527-3ca2bc00bef5')
    def test_get_service_by_service_binary_name(self):
        """Listing nova services by binary name"""
        binary_name = 'nova-compute'
        services = self.client.list_services(binary=binary_name)['services']
        self.assertNotEmpty(services)
        for service in services:
            self.assertEqual(binary_name, service['binary'])

    @decorators.idempotent_id('affb42d5-5b4b-43c8-8b0b-6dca054abcca')
    def test_get_service_by_host_name(self):
        """Listing nova services by host name"""
        services = self.client.list_services()['services']
        host_name = services[0]['host']
        services_on_host = [service for service in services if
                            service['host'] == host_name]

        services = self.client.list_services(host=host_name)['services']

        # we could have a periodic job checkin between the 2 service
        # lookups, so only compare binary lists.
        s1 = map(lambda x: x['binary'], services)
        s2 = map(lambda x: x['binary'], services_on_host)

        # sort the lists before comparing, to take out dependency
        # on order.
        self.assertEqual(sorted(s1), sorted(s2))



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common import api_version_request
from tempest.lib import exceptions
from tempest.tests import base


class APIVersionRequestTests(base.TestCase):
    def test_valid_version_strings(self):
        def _test_string(version, exp_major, exp_minor):
            v = api_version_request.APIVersionRequest(version)
            self.assertEqual(v.ver_major, exp_major)
            self.assertEqual(v.ver_minor, exp_minor)

        _test_string("1.1", 1, 1)
        _test_string("2.10", 2, 10)
        _test_string("5.234", 5, 234)
        _test_string("12.5", 12, 5)
        _test_string("2.0", 2, 0)
        _test_string("2.200", 2, 200)

    def test_null_version(self):
        v = api_version_request.APIVersionRequest()
        self.assertTrue(v.is_null())

    def test_invalid_version_strings(self):
        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "2")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "200")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "2.1.4")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "200.23.66.3")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "5 .3")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "5. 3")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "5.03")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "02.1")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "2.001")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, " 2.1")

        self.assertRaises(exceptions.InvalidAPIVersionString,
                          api_version_request.APIVersionRequest, "2.1 ")

    def test_version_comparisons(self):
        vers2_0 = api_version_request.APIVersionRequest("2.0")
        vers2_5 = api_version_request.APIVersionRequest("2.5")
        vers5_23 = api_version_request.APIVersionRequest("5.23")
        v_null = api_version_request.APIVersionRequest()
        v_latest = api_version_request.APIVersionRequest('latest')

        self.assertTrue(v_null < vers2_5)
        self.assertTrue(vers2_0 < vers2_5)
        self.assertTrue(vers2_0 <= vers2_5)
        self.assertTrue(vers2_0 <= vers2_0)
        self.assertTrue(vers2_5 > v_null)
        self.assertTrue(vers5_23 > vers2_5)
        self.assertTrue(vers2_0 >= vers2_0)
        self.assertTrue(vers5_23 >= vers2_5)
        self.assertTrue(vers2_0 != vers2_5)
        self.assertTrue(vers2_0 == vers2_0)
        self.assertTrue(vers2_0 != v_null)
        self.assertTrue(v_null == v_null)
        self.assertTrue(vers2_0 <= v_latest)
        self.assertTrue(vers2_0 != v_latest)
        self.assertTrue(v_latest == v_latest)
        self.assertRaises(TypeError, vers2_0.__lt__, "2.1")

    def test_version_matches(self):
        vers2_0 = api_version_request.APIVersionRequest("2.0")
        vers2_5 = api_version_request.APIVersionRequest("2.5")
        vers2_45 = api_version_request.APIVersionRequest("2.45")
        vers3_3 = api_version_request.APIVersionRequest("3.3")
        vers3_23 = api_version_request.APIVersionRequest("3.23")
        vers4_0 = api_version_request.APIVersionRequest("4.0")
        v_null = api_version_request.APIVersionRequest()
        v_latest = api_version_request.APIVersionRequest('latest')

        def _check_version_matches(version, version1, version2, check=True):
            if check:
                msg = "Version %s does not matches with [%s - %s] range"
                self.assertTrue(version.matches(version1, version2),
                                msg % (version.get_string(),
                                       version1.get_string(),
                                       version2.get_string()))
            else:
                msg = "Version %s matches with [%s - %s] range"
                self.assertFalse(version.matches(version1, version2),
                                 msg % (version.get_string(),
                                        version1.get_string(),
                                        version2.get_string()))

        _check_version_matches(vers2_5, vers2_0, vers2_45)
        _check_version_matches(vers2_5, vers2_0, v_null)
        _check_version_matches(vers2_0, vers2_0, vers2_5)
        _check_version_matches(vers3_3, vers2_5, vers3_3)
        _check_version_matches(vers3_3, v_null, vers3_3)
        _check_version_matches(vers3_3, v_null, vers4_0)
        _check_version_matches(vers2_0, vers2_5, vers2_45, False)
        _check_version_matches(vers3_23, vers2_5, vers3_3, False)
        _check_version_matches(vers2_5, vers2_45, vers2_0, False)
        _check_version_matches(vers2_5, vers2_0, v_latest)
        _check_version_matches(v_latest, v_latest, v_latest)
        _check_version_matches(vers2_5, v_latest, v_latest, False)
        _check_version_matches(v_latest, vers2_0, vers4_0, False)

        self.assertRaises(ValueError, v_null.matches, vers2_0, vers2_45)

    def test_get_string(self):
        vers_string = ["3.23", "latest"]
        for ver in vers_string:
            ver_obj = api_version_request.APIVersionRequest(ver)
            self.assertEqual(ver, ver_obj.get_string())

        self.assertIsNotNone(
            api_version_request.APIVersionRequest().get_string)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.lib.common import api_version_request
from tempest.lib import exceptions


LATEST_MICROVERSION = 'latest'


class BaseMicroversionTest(object):
    """Mixin class for API microversion test class."""

    # NOTE: Basically, each microversion is small API change and we
    # can use the same tests for most microversions in most cases.
    # So it is nice to define the test class as possible to run
    # for all microversions. We need to define microversion range
    # (min_microversion, max_microversion) on each test class if necessary.
    min_microversion = None
    max_microversion = LATEST_MICROVERSION
    volume_min_microversion = None
    volume_max_microversion = LATEST_MICROVERSION
    placement_min_microversion = None
    placement_max_microversion = LATEST_MICROVERSION


def check_skip_with_microversion(test_min_version, test_max_version,
                                 cfg_min_version, cfg_max_version):
    """Checks API microversions range and returns whether test needs to be skip

    Compare the test and configured microversion range and returns
    whether test microversion range is out of configured one.
    This method can be used to skip the test based on configured and test
    microversion range.

    :param test_min_version: Test Minimum Microversion
    :param test_max_version: Test Maximum Microversion
    :param cfg_min_version: Configured Minimum Microversion
    :param cfg_max_version: Configured Maximum Microversion
    :returns: boolean
    """

    min_version = api_version_request.APIVersionRequest(test_min_version)
    max_version = api_version_request.APIVersionRequest(test_max_version)
    config_min_version = api_version_request.APIVersionRequest(cfg_min_version)
    config_max_version = api_version_request.APIVersionRequest(cfg_max_version)
    if ((min_version > max_version) or
        (config_min_version > config_max_version)):
        msg = ("Test Class versions [%s - %s]. "
               "Configuration versions [%s - %s]."
               % (min_version.get_string(),
                  max_version.get_string(),
                  config_min_version.get_string(),
                  config_max_version.get_string()))
        raise exceptions.InvalidAPIVersionRange(msg)

    # NOTE: Select tests which are in range of configuration like
    #               config min           config max
    # ----------------+--------------------------+----------------
    # ...don't-select|
    #            ...select...  ...select...  ...select...
    #                                             |don't-select...
    # ......................select............................
    if (max_version < config_min_version or
        config_max_version < min_version):
        msg = ("The microversion range[%s - %s] of this test is out of the "
               "configuration range[%s - %s]."
               % (min_version.get_string(),
                  max_version.get_string(),
                  config_min_version.get_string(),
                  config_max_version.get_string()))
        raise testtools.TestCase.skipException(msg)


def select_request_microversion(test_min_version, cfg_min_version):
    """Select microversion from test and configuration min version.

    Compare requested microversion and return the maximum
    microversion out of those.

    :param test_min_version: Test Minimum Microversion
    :param cfg_min_version: Configured Minimum Microversion
    :returns: Selected microversion string
    """

    test_version = api_version_request.APIVersionRequest(test_min_version)
    cfg_version = api_version_request.APIVersionRequest(cfg_min_version)
    max_version = cfg_version if cfg_version >= test_version else test_version
    return max_version.get_string()


def assert_version_header_matches_request(api_microversion_header_name,
                                          api_microversion,
                                          response_header):
    """Checks API microversion in response header

    Verify whether microversion is present in response header
    and with specified 'api_microversion' value.

    :param api_microversion_header_name: Microversion header name
            Example- "X-OpenStack-Nova-API-Version"
    :param api_microversion: Microversion number like "2.10", type str.
    :param response_header: Response header where microversion is
            expected to be present.
    """
    if not isinstance(api_microversion, str):
        raise TypeError('api_microversion must be a string')
    api_microversion_header_name = api_microversion_header_name.lower()
    if (api_microversion_header_name not in response_header or
        api_microversion != response_header[api_microversion_header_name]):
        msg = ("Microversion header '%s' with value '%s' does not match in "
               "response - %s. " % (api_microversion_header_name,
                                    api_microversion,
                                    response_header))
        raise exceptions.InvalidHTTPResponseHeader(msg)


def compare_version_header_to_response(api_microversion_header_name,
                                       api_microversion,
                                       response_header,
                                       operation='eq'):
    """Compares API microversion in response header to ``api_microversion``.

    Compare the ``api_microversion`` value in response header if microversion
    header is present in response, otherwise return false.

    To make this function work for APIs which do not return microversion
    header in response (example compute v2.0), this function does *not* raise
    InvalidHTTPResponseHeader.

    :param api_microversion_header_name: Microversion header name. Example:
        'Openstack-Api-Version'.
    :param api_microversion: Microversion number. Example:

        * '2.10' for the old-style header name, 'X-OpenStack-Nova-API-Version'
        * 'Compute 2.10' for the new-style header name, 'Openstack-Api-Version'

    :param response_header: Response header where microversion is
        expected to be present.
    :param operation: The boolean operation to use to compare the
        ``api_microversion`` to the microversion in ``response_header``.
        Can be 'lt', 'eq', 'gt', 'le', 'ne', 'ge'. Default is 'eq'. The
        operation type should be based on the order of the arguments:
        ``api_microversion`` <operation> ``response_header`` microversion.
    :returns: True if the comparison is logically true, else False if the
        comparison is logically false or if ``api_microversion_header_name`` is
        missing in the ``response_header``.
    :raises InvalidParam: If the operation is not lt, eq, gt, le, ne or ge.
    """
    api_microversion_header_name = api_microversion_header_name.lower()
    if api_microversion_header_name not in response_header:
        return False

    op = getattr(api_version_request.APIVersionRequest,
                 '__%s__' % operation, None)

    if op is None:
        msg = ("Operation %s is invalid. Valid options include: lt, eq, gt, "
               "le, ne, ge." % operation)
        raise exceptions.InvalidParam(invalid_param=msg)

    # Remove "volume" from "volume <microversion>", for example, so that the
    # microversion can be converted to `APIVersionRequest`.
    api_version = api_microversion.split(' ')[-1]
    resp_version = response_header[api_microversion_header_name].split(' ')[-1]
    if not op(
        api_version_request.APIVersionRequest(api_version),
        api_version_request.APIVersionRequest(resp_version)):
        return False

    return True



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_33 \
    import hypervisors as hypervisorsv233

###########################################################################
#
# 2.53:
#
# servers parameter is added to the response body for the following
# APIs:
#
# - GET /os-hypervisor
# - GET /os-hypervisors
# - GET /os-hypervisors/detail
#
###########################################################################

servers = {
    'type': 'array',
    'items': {
        'type': 'object',
        'properties': {
            'uuid': {'type': 'string', 'format': 'uuid'},
            'name': {'type': 'string'},
        },
        'additionalProperties': False,
    },
}

hypervisor_detail = copy.deepcopy(hypervisorsv233.hypervisor_detail)
hypervisor_detail['properties'].update({'servers': servers})
get_hypervisor = copy.deepcopy(hypervisorsv233.get_hypervisor)
get_hypervisor['response_body']['properties'].update(
    {'hypervisor': hypervisor_detail})
list_hypervisors_detail = copy.deepcopy(
    hypervisorsv233.list_hypervisors_detail)
list_hypervisors_detail['response_body']['properties']['hypervisors'].update(
    {'items': hypervisor_detail})

list_search_hypervisors = copy.deepcopy(
    hypervisorsv233.list_search_hypervisors)
list_search_hypervisors['response_body']['properties']['hypervisors'][
    'items']['properties'].update({'servers': servers})

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.33 ***
get_hypervisor_statistics = \
    copy.deepcopy(hypervisorsv233.get_hypervisor_statistics)
get_hypervisor_uptime = copy.deepcopy(hypervisorsv233.get_hypervisor_uptime)
get_hypervisors_servers = copy.deepcopy(
    hypervisorsv233.get_hypervisors_servers)



# Copyright 2020 Samsung Electronics Co., Ltd
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class IdentityProvidersClient(rest_client.RestClient):

    def register_identity_provider(self, identity_provider_id, **kwargs):
        """Register an identity provider.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#register-an-identity-provider
        """
        post_body = json.dumps({'identity_provider': kwargs})
        resp, body = self.put(
            'OS-FEDERATION/identity_providers/%s' % identity_provider_id,
            post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_identity_providers(self, **params):
        """List identity providers.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#list-identity-providers
        """
        url = 'identity_providers'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_identity_provider(self, identity_provider_id):
        """Get identity provider.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#get-identity-provider
        """
        resp, body = self.get(
            'OS-FEDERATION/identity_providers/%s' % identity_provider_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_identity_provider(self, identity_provider_id):
        """Delete identity provider.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#delete-identity-provider
        """
        resp, body = self.delete(
            'OS-FEDERATION/identity_providers/%s' % identity_provider_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_identity_provider(self, identity_provider_id, **kwargs):
        """Update identity provider.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#update-identity-provider
        """
        post_body = json.dumps({'identity_provider': kwargs})
        resp, body = self.patch(
            'OS-FEDERATION/identity_providers/%s' % identity_provider_id,
            post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


# NOTE(zhufl): This inherits from BaseVolumeAdminTest because
# it requires force_tenant_isolation=True, which need admin
# credentials to create non-admin users for the tests.
class AbsoluteLimitsTests(base.BaseVolumeAdminTest):  # noqa: T115
    """Test volume absolute limits"""

    # avoid existing volumes of pre-defined tenant
    force_tenant_isolation = True

    @classmethod
    def resource_setup(cls):
        super(AbsoluteLimitsTests, cls).resource_setup()

        # Create a shared volume for tests
        cls.volume = cls.create_volume()

    @classmethod
    def skip_checks(cls):
        super(AbsoluteLimitsTests, cls).skip_checks()
        if not CONF.auth.use_dynamic_credentials:
            raise cls.skipException("Must use dynamic credentials.")

    @decorators.idempotent_id('8e943f53-e9d6-4272-b2e9-adcf2f7c29ad')
    def test_get_volume_absolute_limits(self):
        """Test getting volume absolute limits"""
        absolute_limits = \
            self.volume_limits_client.show_limits(
            )['limits']['absolute']

        # verify volume limits and defaults per tenants
        self.assertEqual(absolute_limits['totalGigabytesUsed'],
                         CONF.volume.volume_size)
        self.assertEqual(absolute_limits['totalVolumesUsed'], 1)
        self.assertEqual(absolute_limits['totalSnapshotsUsed'], 0)
        self.assertEqual(absolute_limits['totalBackupsUsed'], 0)
        self.assertEqual(absolute_limits['totalBackupGigabytesUsed'], 0)



# Copyright 2021 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import loggable_resource_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestLoggableResourceClient(base.BaseServiceTest):

    FAKE_LOGS = {
        "loggable_resources": [
            {
                "type": "security_group"
            },
            {
                "type": "none"
            }
        ]
    }

    def setUp(self):
        super(TestLoggableResourceClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.loggable_resource_client = \
            loggable_resource_client.LoggableResourceClient(
                fake_auth, "network", "regionOne")

    def _test_list_loggable_resources(self, bytes_body=False):
        self.check_service_client_function(
            self.loggable_resource_client.list_loggable_resources,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_LOGS,
            bytes_body,
            200)

    def test_list_loggable_resources_with_str_body(self):
        self._test_list_loggable_resources()

    def test_list_loggable_resources_with_bytes_body(self):
        self._test_list_loggable_resources(bytes_body=True)



# Copyright 2017 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.compute.v2_47 import servers as servers247


show_server_diagnostics = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'state': {
                'type': 'string', 'enum': [
                    'pending', 'running', 'paused', 'shutdown', 'crashed',
                    'suspended']
            },
            'driver': {
                'type': 'string', 'enum': [
                    'libvirt', 'xenapi', 'vmwareapi', 'ironic', 'hyperv']
            },
            'hypervisor': {'type': ['string', 'null']},
            'hypervisor_os': {'type': ['string', 'null']},
            'uptime': {'type': ['integer', 'null']},
            'config_drive': {'type': 'boolean'},
            'num_cpus': {'type': 'integer'},
            'num_nics': {'type': 'integer'},
            'num_disks': {'type': 'integer'},
            'memory_details': {
                'type': 'object',
                'properties': {
                    'maximum': {'type': ['integer', 'null']},
                    'used': {'type': ['integer', 'null']}
                },
                'additionalProperties': False,
                'required': ['maximum', 'used']
            },
            'cpu_details': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': ['integer', 'null']},
                        'time': {'type': ['integer', 'null']},
                        'utilisation': {'type': ['integer', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['id', 'time', 'utilisation']
                }
            },
            'nic_details': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'mac_address': {'oneOf': [parameter_types.mac_address,
                                                  {'type': 'null'}]},
                        'rx_octets': {'type': ['integer', 'null']},
                        'rx_errors': {'type': ['integer', 'null']},
                        'rx_drop': {'type': ['integer', 'null']},
                        'rx_packets': {'type': ['integer', 'null']},
                        'rx_rate': {'type': ['integer', 'null']},
                        'tx_octets': {'type': ['integer', 'null']},
                        'tx_errors': {'type': ['integer', 'null']},
                        'tx_drop': {'type': ['integer', 'null']},
                        'tx_packets': {'type': ['integer', 'null']},
                        'tx_rate': {'type': ['integer', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['mac_address', 'rx_octets', 'rx_errors',
                                 'rx_drop',
                                 'rx_packets', 'rx_rate', 'tx_octets',
                                 'tx_errors',
                                 'tx_drop', 'tx_packets', 'tx_rate']
                }
            },
            'disk_details': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'read_bytes': {'type': ['integer', 'null']},
                        'read_requests': {'type': ['integer', 'null']},
                        'write_bytes': {'type': ['integer', 'null']},
                        'write_requests': {'type': ['integer', 'null']},
                        'errors_count': {'type': ['integer', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['read_bytes', 'read_requests', 'write_bytes',
                                 'write_requests', 'errors_count']
                }
            }
        },
        'additionalProperties': False,
        'required': [
            'state', 'driver', 'hypervisor', 'hypervisor_os', 'uptime',
            'config_drive', 'num_cpus', 'num_nics', 'num_disks',
            'memory_details', 'cpu_details', 'nic_details', 'disk_details'],
    }
}

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
list_servers = copy.deepcopy(servers247.list_servers)
get_remote_consoles = copy.deepcopy(servers247.get_remote_consoles)
list_tags = copy.deepcopy(servers247.list_tags)
update_all_tags = copy.deepcopy(servers247.update_all_tags)
delete_all_tags = copy.deepcopy(servers247.delete_all_tags)
check_tag_existence = copy.deepcopy(servers247.check_tag_existence)
update_tag = copy.deepcopy(servers247.update_tag)
delete_tag = copy.deepcopy(servers247.delete_tag)
get_server = copy.deepcopy(servers247.get_server)
list_servers_detail = copy.deepcopy(servers247.list_servers_detail)
update_server = copy.deepcopy(servers247.update_server)
rebuild_server = copy.deepcopy(servers247.rebuild_server)
rebuild_server_with_admin_pass = copy.deepcopy(
    servers247.rebuild_server_with_admin_pass)
attach_volume = copy.deepcopy(servers247.attach_volume)
show_volume_attachment = copy.deepcopy(servers247.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers247.list_volume_attachments)
show_instance_action = copy.deepcopy(servers247.show_instance_action)
create_backup = copy.deepcopy(servers247.create_backup)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.security_groups import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class SecurityGroupRulesNegativeTestJSON(base.BaseSecurityGroupsTest):
    """Negative tests of security group rules API

    Negative tests of security group rules API with compute microversion
    less than 2.36.
    """

    @classmethod
    def setup_clients(cls):
        super(SecurityGroupRulesNegativeTestJSON, cls).setup_clients()
        cls.rules_client = cls.security_group_rules_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1d507e98-7951-469b-82c3-23f1e6b8c254')
    def test_create_security_group_rule_with_non_existent_id(self):
        """Test creating security group rule with non existent parent group

        Negative test: Creation of security group rule should fail
        with non existent parent group id.
        """
        # Adding rules to the non existent Security Group id
        parent_group_id = self.generate_random_security_group_id()
        ip_protocol = 'tcp'
        from_port = 22
        to_port = 22
        self.assertRaises(lib_exc.NotFound,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('2244d7e4-adb7-4ecb-9930-2d77e123ce4f')
    def test_create_security_group_rule_with_invalid_id(self):
        """Test creating security group rule with invalid parent group id

        Negative test: Creation of security group rule should fail
        with parent group id which is not integer.
        """
        # Adding rules to the non int Security Group id
        parent_group_id = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='non_int_id')
        ip_protocol = 'tcp'
        from_port = 22
        to_port = 22
        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8bd56d02-3ffa-4d67-9933-b6b9a01d6089')
    def test_create_security_group_rule_duplicate(self):
        """Test creating duplicate security group rule should fail"""
        # Creating a Security Group to add rule to it
        sg = self.create_security_group()
        # Adding rules to the created Security Group
        parent_group_id = sg['id']
        ip_protocol = 'tcp'
        from_port = 22
        to_port = 22

        rule = self.rules_client.create_security_group_rule(
            parent_group_id=parent_group_id, ip_protocol=ip_protocol,
            from_port=from_port, to_port=to_port)['security_group_rule']
        self.addCleanup(self.rules_client.delete_security_group_rule,
                        rule['id'])
        # Add the same rule to the group should fail
        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('84c81249-9f6e-439c-9bbf-cbb0d2cddbdf')
    def test_create_security_group_rule_with_invalid_ip_protocol(self):
        """Test creating security group rule with invalid ip protocol

        Negative test: Creation of security group rule should fail
        with invalid ip_protocol.
        """
        # Creating a Security Group to add rule to it
        sg = self.create_security_group()
        # Adding rules to the created Security Group
        parent_group_id = sg['id']
        ip_protocol = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='999')
        from_port = 22
        to_port = 22

        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('12bbc875-1045-4f7a-be46-751277baedb9')
    def test_create_security_group_rule_with_invalid_from_port(self):
        """Test creating security group rule with invalid from_port

        Negative test: Creation of security group rule should fail
        with invalid from_port.
        """
        # Creating a Security Group to add rule to it
        sg = self.create_security_group()
        # Adding rules to the created Security Group
        parent_group_id = sg['id']
        ip_protocol = 'tcp'
        from_port = data_utils.rand_int_id(start=65536)
        to_port = 22
        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ff88804d-144f-45d1-bf59-dd155838a43a')
    def test_create_security_group_rule_with_invalid_to_port(self):
        """Test creating security group rule with invalid to_port

        Negative test: Creation of security group rule should fail
        with invalid to_port.
        """
        # Creating a Security Group to add rule to it
        sg = self.create_security_group()
        # Adding rules to the created Security Group
        parent_group_id = sg['id']
        ip_protocol = 'tcp'
        from_port = 22
        to_port = data_utils.rand_int_id(start=65536)
        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=parent_group_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('00296fa9-0576-496a-ae15-fbab843189e0')
    def test_create_security_group_rule_with_invalid_port_range(self):
        """Test creating security group rule with invalid port range

        Negative test: Creation of security group rule should fail
        with invalid port range.
        """
        # Creating a Security Group to add rule to it.
        sg = self.create_security_group()
        # Adding a rule to the created Security Group
        secgroup_id = sg['id']
        ip_protocol = 'tcp'
        from_port = 22
        to_port = 21
        self.assertRaises(lib_exc.BadRequest,
                          self.rules_client.create_security_group_rule,
                          parent_group_id=secgroup_id,
                          ip_protocol=ip_protocol, from_port=from_port,
                          to_port=to_port)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('56fddcca-dbb8-4494-a0db-96e9f869527c')
    def test_delete_security_group_rule_with_non_existent_id(self):
        """Test deleting non existent security group rule should fail"""
        non_existent_rule_id = self.generate_random_security_group_id()
        self.assertRaises(lib_exc.NotFound,
                          self.rules_client.delete_security_group_rule,
                          non_existent_rule_id)
